==> ideascloud-sdk-1.2.0.tgz: package/classes/DevCommunityTools/index.js <==
const callService = require('../../tools/callService');
const fs = require('fs');
const path = require('path');
const validateJSONSchema = require('jsonschema').validate;
const axios = require('axios').default;
const zipdir = require('zip-dir');
const mime = require('mime-types');
const AdmZip = require('adm-zip');

class DevCommunityTools {

  /**
   * 
   * @param {Object} params
   * @param {Object} [params.accessToken] - Personal Access Token
   */
  constructor (params = {}) {
    this._accessToken = params.accessToken || process.env.IC_ACCESS_TOKEN;
  }

  /**
   * @returns {Promise} Update webapp template setup and source code
   */
  async updateWebappTemplateSetupAndSource (webappPath) {

    // Init setup
    const webappFullPath = path.resolve(webappPath);
    const webappICTemplateDirPath = path.join(webappFullPath, '.ic-template');
    const webappICTemplateSetupFilePath = path.join(webappICTemplateDirPath, 'setup.json');
    const webappICTemplateSetupFileJSON = JSON.parse(fs.readFileSync(webappICTemplateSetupFilePath, 'utf-8'));

    // Validation
    const validationErrors = validateJSONSchema(webappICTemplateSetupFileJSON, {
      type: 'object',
      properties: {
        id: { type: 'string' },
        sourcePath: { type: 'string' },
        setup: {
          type: 'object',
          properties: {
            technologies: { type: 'array' },
            structureSchema: { type: 'object' },
            snippets: { type: 'array' },
            actions: { type: 'array' },
          },
          required: ['technologies', 'structureSchema', 'snippets', 'actions']
        }
      },
      required: ['id', 'sourcePath', 'setup']
    }).errors;
    if (validationErrors.length) {
      throw `\n\nValidation error: \n\n${JSON.stringify(validationErrors, null, 2)}\n\n`;
    }
    const sourceIndexContent = fs.readFileSync(path.join(webappFullPath, webappICTemplateSetupFileJSON.sourcePath, 'index.html'), 'utf-8');
    if (!sourceIndexContent.match(/html/i)) {
      throw 'Invalid template source: index.html file is malformed';
    }
    let previewFileName = fs.readdirSync(webappICTemplateDirPath).filter(filename => !!filename.match(/^preview\.[a-z]+$/))[0]
    if (!previewFileName) {
      throw 'Preview file not found in .ic-template dir';
    }
    let previewMimeType = mime.lookup(previewFileName);
    let isValidMimeType = ({
      'image/jpeg': true,
      'image/png': true,
      'image/gif': true,
      'image/webp': true,
    })[previewMimeType];
    if (!isValidMimeType) {
      throw `Invalid preview file type: ${previewMimeType || 'unknown'}`
    }
    
    // Upload setup and request upload url
    let uploadReqOutput = await callService('updateWebappTemplateSetupAndRequestUploadUrl', {
      accessToken: this._accessToken,
      templateId: webappICTemplateSetupFileJSON.id,
      setup: webappICTemplateSetupFileJSON
    })
    let zipBuffer = await zipdir(path.join(webappFullPath, webappICTemplateSetupFileJSON.sourcePath));
    let admzip = new AdmZip(zipBuffer);
    let previewImageBuffer = fs.readFileSync(path.join(webappICTemplateDirPath, previewFileName));
    admzip.addFile('.ic-preview', previewImageBuffer);

    await axios({
      url: uploadReqOutput.url,
      method: 'PUT',
      headers: {
        'Content-Type': 'application/zip'
      },
      data: admzip.toBuffer()
    })

  }

}

module.exports = DevCommunityTools;

==> ideascloud-sdk-1.2.3.tgz: package/classes/DevCommunityTools/index.js <==
const callService = require('../../tools/callService');
const fs = require('fs');
const path = require('path');
const validateJSONSchema = require('jsonschema').validate;
const axios = require('axios').default;
const zipdir = require('zip-dir');
const mime = require('mime-types');
const AdmZip = require('adm-zip');

class DevCommunityTools {

  /**
   * 
   * @param {Object} params
   * @param {Object} [params.accessToken] - Personal Access Token
   */
  constructor (params = {}) {
    this._accessToken = params.accessToken || process.env.IC_ACCESS_TOKEN;
  }

  /**
   * @returns {Promise} Update webapp template setup and source code
   */
  async updateWebappTemplateSetupAndSource (webappPath) {

    // Init setup
    const webappFullPath = path.resolve(webappPath);
    const webappICTemplateDirPath = path.join(webappFullPath, '.ic-template');
    const webappICTemplateSetupFilePath = path.join(webappICTemplateDirPath, 'setup.json');
    const webappICTemplateSetupFileJSON = JSON.parse(fs.readFileSync(webappICTemplateSetupFilePath, 'utf-8'));

    // Validation
    const validationErrors = validateJSONSchema(webappICTemplateSetupFileJSON, {
      type: 'object',
      properties: {
        id: { type: 'string' },
        sourcePath: { type: 'string' },
        setup: {
          type: 'object',
          properties: {
            technologies: { type: 'array' },
            structureSchema: { type: 'object' },
            snippets: { type: 'array' },
            actions: { type: 'array' },
          },
          required: ['technologies', 'structureSchema', 'snippets', 'actions']
        }
      },
      required: ['id', 'sourcePath', 'setup']
    }).errors;
    if (validationErrors.length) {
      throw `\n\nValidation error: \n\n${JSON.stringify(validationErrors, null, 2)}\n\n`;
    }
    const sourceIndexContent = fs.readFileSync(path.join(webappFullPath, webappICTemplateSetupFileJSON.sourcePath, 'index.html'), 'utf-8');
    if (!sourceIndexContent.match(/html/i)) {
      throw 'Invalid template source: index.html file is malformed';
    }
    let previewFileName = fs.readdirSync(webappICTemplateDirPath).filter(filename => !!filename.match(/^preview\.[a-z]+$/))[0]
    if (!previewFileName) {
      throw 'Preview file not found in .ic-template dir';
    }
    let previewMimeType = mime.lookup(previewFileName);
    let isValidMimeType = ({
      'image/jpeg': true,
      'image/png': true,
      'image/gif': true,
      'image/webp': true,
    })[previewMimeType];
    if (!isValidMimeType) {
      throw `Invalid preview file type: ${previewMimeType || 'unknown'}`
    }
    
    // Upload setup and request upload url
    let uploadReqOutput = await callService('updateWebappTemplateSetupAndRequestUploadUrl', {
      accessToken: this._accessToken,
      templateId: webappICTemplateSetupFileJSON.id,
      setup: webappICTemplateSetupFileJSON
    })
    let zipBuffer = await zipdir(path.join(webappFullPath, webappICTemplateSetupFileJSON.sourcePath));
    let rawWebappSourceBufferZip = await zipdir(webappFullPath);
    let admzip = new AdmZip(zipBuffer);
    let previewImageBuffer = fs.readFileSync(path.join(webappICTemplateDirPath, previewFileName));
    admzip.addFile('.ic-preview', previewImageBuffer);
    admzip.addFile('.raw-template-source.zip', rawWebappSourceBufferZip);

    await axios({
      url: uploadReqOutput.url,
      method: 'PUT',
      headers: {
        'Content-Type': 'application/zip'
      },
      data: admzip.toBuffer(),
      maxContentLength: Infinity,
      maxBodyLength: Infinity
    })

  }

}

module.exports = DevCommunityTools;
